pub mod svg;
use svg::*;

pub mod key;
use key::*;

use askama::Template;
use derive_more::From;
use std::{fs::OpenOptions, io::Write};

#[derive(Template)]
#[template(path = "document.xml")]
struct DocumentTemplate<'a> {
    dimensions: &'a DocumentDimensions,
    box_size: SVGMeasure,
    positions: Vec<(&'a Key, (SVGMeasure, SVGMeasure))>,
}

struct DocumentDimensions {
    width: SVGMeasure,
    height: SVGMeasure,
}

#[derive(From, Debug)]
enum Error {
    Io(std::io::Error),
    Template(askama::Error),
}

fn positions(
    size: SVGMeasure,
    padding: SVGMeasure,
    dimensions: &DocumentDimensions,
) -> Vec<(&Key, (SVGMeasure, SVGMeasure))> {
    let grid = size + padding;
    let row_count = ((dimensions.width - padding * 2.0) / grid) as u32;
    OYAYUBI
        .iter()
        .enumerate()
        .map(|(i, key)| (
            key,
            (
                grid * (i as u32 % row_count).into() + padding,
                grid * (i as u32 / row_count).into() + padding,
            )
        ))
        .collect()
}

fn main() -> Result<(), Error> {
    let mut file = OpenOptions::new()
        .write(true)
        .truncate(true)
        .create(true)
        .open("output.svg")?;
    let dimensions = DocumentDimensions {
        width: SVGMeasure::new(8.5, SVGUnit::Inch),
        height: SVGMeasure::new(11.0, SVGUnit::Inch),
    };
    let box_size = SVGMeasure::new(14.0, SVGUnit::Millimeter);
    let padding = SVGMeasure::new(2.5, SVGUnit::Millimeter);
    let document = DocumentTemplate {
        positions: positions(box_size, padding, &dimensions),
        dimensions: &dimensions,
        box_size,
    };
    write!(file, "{}", document.render()?)?;
    Ok(())
}
